@@ -1,6 +1,6 @@
 ###           groupedData - data frame with a grouping structure
 ###
-### Copyright 2006-2018  The R Core team
+### Copyright 2006-2023  The R Core team
 ### Copyright 1997-2003  Jose C. Pinheiro,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
 #
@@ -35,9 +35,9 @@ groupedData <-
     stop("right-hand side of first argument must be a conditional expression")
   }
   mCall <- match.call()
-  mCall[[1]] <- if(length(grpForm) == 1)## nlme::: needed if 'nlme' not attached
-		     quote(nlme:::nfGroupedData)
-		else quote(nlme:::nmGroupedData)
+  mCall[[1]] <- if(length(grpForm) == 1)## nlme:: needed if 'nlme' not attached
+		     quote(nlme::nfGroupedData)
+		else quote(nlme::nmGroupedData)
   eval(mCall, envir = parent.frame())
 }
 
@@ -49,14 +49,12 @@ nfGroupedData <-
 	   FUN = function(x) max(x, na.rm = TRUE), outer = NULL,
            inner = NULL, labels = NULL, units = NULL)
 {
-  ## want to stop exporting undocumented(!) nmGroupedData()
-  if(!local({ N <- -1L
-      while(identical(sys.call(N)[[1]], quote(eval))) N <- N-1L
-      identical(.sf <- sys.function(N), groupedData) ||
-          identical(environment(.sf), .ns) ||
-          identical(environment(sys.function(N-1L)), .ns) ||
-          identical(environment(sys.function(N-2L)), .ns)
-  }))
+  ## want to stop exporting nfGroupedData()
+  ## called internally from groupedData() via eval or from collapse.groupedData()
+  .internal <- any(unlist(lapply(tail(sys.frames()[-sys.nframe()],
+                                      3), # no need to look further up
+                                 utils::packageName)) == "nlme")
+  if(!.internal)
     .Deprecated("groupedData", "nlme")
   if (!(inherits(formula, "formula") && length(formula) == 3)) {
     stop("first argument to 'nfGroupedData' must be a two-sided formula")
@@ -115,7 +113,7 @@ nfGroupedData <-
   attr( data, "order.groups" ) <- order.groups
   cl <-
       if ((length(all.vars(getCovariateFormula(formula))) == 0) ||
-          (data.class(primary) != "numeric")) {
+          !is.numeric(primary)) {
           "nffGroupedData"  # primary covariate is a factor or a "1"
       } else {
           "nfnGroupedData"  # primary covariate is numeric
@@ -133,14 +131,12 @@ nmGroupedData <-
 	   FUN = function(x) max(x, na.rm = TRUE), outer = NULL,
            inner = NULL, labels = NULL, units = NULL)
 {
-  ## want to stop exporting undocumented(!) nmGroupedData()
-  if(!local({ N <- -1L
-      while(identical(sys.call(N)[[1]], quote(eval))) N <- N-1L
-      identical(.sf <- sys.function(N), groupedData) ||
-          identical(environment(.sf), .ns) ||
-          identical(environment(sys.function(N-1L)), .ns) ||
-          identical(environment(sys.function(N-2L)), .ns)
-  }))
+  ## want to stop exporting nmGroupedData()
+  ## called internally from groupedData() via eval
+  .internal <- any(unlist(lapply(tail(sys.frames()[-sys.nframe()],
+                                      3), # no need to look further up
+                                 utils::packageName)) == "nlme")
+  if(!.internal)
     .Deprecated("groupedData", "nlme")
   if (!(inherits(formula, "formula") && length(formula) == 3))
     stop("first argument to 'nmGroupedData' must be a two-sided formula")
@@ -232,11 +228,11 @@ collapse.groupedData <-
     if (!is.list(subset)) {
       stop("'subset' must be a list")
     }
-    if (!any(is.na(match(names(subset), 1:Q)))) {
+    if (!anyNA(match(names(subset), 1:Q))) {
       ## subset names given as integers
       names(subset) <- grpNames[names(subset)]
     }
-    if (any(is.na(match(names(subset), grpNames)))) {
+    if (anyNA(match(names(subset), grpNames))) {
       stop("undefined group declared in 'subset'")
     }
     auxSubset <- rep(TRUE, dim(object)[1])
@@ -584,7 +580,7 @@ update.groupedData <-
 }
 
 "[.groupedData" <-
-  function(x, i, j, drop = if (missing(i)) TRUE else length(cols) == 1)
+  function(x, i, j, drop = missing(i) || (nargs() >= 3L && length(cols) == 1))
 {
   oAttr <- attributes(x)
   x <- as.data.frame(x)
@@ -593,16 +589,16 @@ update.groupedData <-
   allV <- all.vars(asOneFormula(oAttr[["formula"]], oAttr[["inner"]],
                                 oAttr[["outer"]]))
   ## check if any columns used in formulas were deleted
-  if( any( is.na( match( allV, names(data) ) ) ) ) { # return data frame
+  if(anyNA(match(allV, names(data)))) { # return data frame
     cols <- ncol(data)
-    return( data[, seq(length=ncol(data)), drop = drop] )
+    return(data[, seq_len(cols), drop = drop])
   }
   args <- as.list(oAttr)
   args <- args[ is.na( match( names( args ), c( "names", "row.names" ) ) ) ]
   if (nrow(x) == nrow(data)) {		# only columns deleted
     attributes(data) <- c( attributes( data ), args )
     return( data )
-  }
+  } ## otherwise, return  "groupedData" :
   ## pruning the levels of factors
   whichFact <- unlist(lapply(data, is.factor))
   data[whichFact] <- lapply(data[whichFact], function(x) x[drop = TRUE])
@@ -643,7 +639,7 @@ balancedGrouped <-
   form <- as.formula( form )
   data <- t( as.matrix( data ) )
   dn <- dimnames( data )
-  if ( all( !is.na( as.numeric( dn[[1]] ) ) ) ) {
+  if ( !anyNA( suppressWarnings( as.numeric( dn[[1]] ) ) ) ) {
     dn[[1]] <- as.numeric( dn[[1]] )
   }
   names(dn) <- c( as.character(getCovariateFormula(form)[[2]]),
